"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrap = exports.RouterService = exports.uwuInjectable = exports.uwuRouter = exports.Inject = exports.Input = exports.uwuTsxController = exports.uwuController = exports.uwuModule = exports.declarationCheck = exports.uwuControllerManager = void 0;
const tsyringe_1 = require("tsyringe");
const core_engine_1 = require("./core.engine");
const core_enginetsx_1 = require("./core.enginetsx");
const core_types_1 = require("./core.types");
const funcs_1 = require("./funcs");
class uwuModuleManager {
    constructor(target) {
        this.target = target;
    }
    ;
    getRouter() {
        const imports = this.target.prototype.imports;
        if (Array.isArray(imports)) {
            for (const obj of imports) {
                if (declarationCheck(obj.prototype.type, core_types_1.uwuDeclaration.Router)) {
                    return obj;
                }
            }
        }
        return null;
    }
}
class uwuRouteManager {
    getModuleControllerByUrl() {
        const parsedUrl = this.processUrl();
        return this.getComponentToRenderByParsedUrl(parsedUrl);
    }
    processUrl() {
        const location = window.location;
        return {
            url: new URL(window.location.toString()),
            params: (0, funcs_1.parseQuery)(location.search),
            path_raw: location.pathname,
            path: location.pathname.split('/').filter(el => el.trim() != ''),
        };
    }
    getComponentToRenderByParsedUrl(parsedUrl) {
        let currentModule = window.root;
        let currentPath = parsedUrl.path.length > 0 ? parsedUrl.path : [''];
        while (currentModule != null) {
            const router = this.getRouterByModule(currentModule);
            if (router == null) {
                throw new Error(`Module with name ${currentModule.name} doesn't have router module in its imports section!`);
            }
            const routes = router.prototype.routes;
            for (let path_section of currentPath) {
                if (path_section == '/') {
                    path_section = '';
                }
                const found_section = routes.find(route => {
                    if (route.path == path_section) {
                        return true;
                    }
                    if (route.regexp) {
                        const regexp = new RegExp(path_section);
                        return regexp.test(route.path);
                    }
                    return false;
                });
                if (found_section == undefined) {
                    throw new Error(`Can't find route target for route '${parsedUrl.path_raw}' at section '${path_section}'`);
                }
                const target = found_section.target;
                if (declarationCheck(target.prototype.type, core_types_1.uwuDeclaration.Controller)) {
                    return [currentModule, target];
                }
                else if (declarationCheck(target.prototype.type, core_types_1.uwuDeclaration.Module)) {
                    currentPath.shift();
                    currentModule = target;
                    break;
                }
                else {
                    throw new Error("Object must be uwu type (uwuModule, uwuService etc.)");
                }
            }
        }
        throw new Error(`Can't find any endpoint for route ${parsedUrl.path_raw}`);
    }
    getRouterByModule(module) {
        const imports = module.prototype.imports;
        if (Array.isArray(imports)) {
            for (const obj of imports) {
                if (declarationCheck(obj.prototype.type, core_types_1.uwuDeclaration.Router)) {
                    return obj;
                }
            }
        }
        return null;
    }
    static generatePageByCurrentUrl() {
        const routeManager = new uwuRouteManager();
        const [module, controller] = routeManager.getModuleControllerByUrl();
        const controllerManager = new uwuControllerManager();
        let [controllerInstance, parentNode] = controllerManager.processPageByController(controller);
        const bindingEngine = new core_engine_1.BindingEngine(controllerInstance);
        [controllerInstance, parentNode] = bindingEngine.getControllerBindedToNodes(controllerInstance, parentNode);
        const body = document.querySelector("body");
        while (body.firstChild) {
            body.removeChild(body.firstChild);
        }
        body.appendChild(parentNode);
    }
}
class uwuControllerManager {
    // This should return instance of controller and binded to it nodes
    processPageByController(controller, entryElement = null) {
        this.updateStyles(controller);
        if (declarationCheck(controller.prototype.type, core_types_1.uwuDeclaration.Controller)) {
            [controller, entryElement] = this.processController(controller, entryElement);
        }
        else {
            [controller, entryElement] = this.processTsxController(controller, entryElement);
        }
        return [controller, entryElement];
    }
    processController(controller, entryElement = null) {
        var _a;
        const content = this.getTemplateContent(controller.prototype.template);
        const rootNode = ((_a = entryElement === null || entryElement === void 0 ? void 0 : entryElement.cloneNode(false)) !== null && _a !== void 0 ? _a : document.createElement(controller.prototype.selector));
        rootNode.guid = (0, funcs_1.guid)();
        rootNode.insertAdjacentHTML("afterbegin", content);
        const controllerInstance = new controller();
        return [controllerInstance, rootNode];
    }
    processTsxController(controller, entryElement = null) {
        var _a;
        const rootNode = ((_a = entryElement === null || entryElement === void 0 ? void 0 : entryElement.cloneNode(false)) !== null && _a !== void 0 ? _a : document.createElement(controller.prototype.selector));
        const controllerInstance = new controller();
        let element = controller.prototype.hook();
        core_enginetsx_1.uwuTsxRenderManager.hooks[controller.prototype.guid] = (newState) => {
            const newHook = controller.prototype.hook(newState);
            rootNode.replaceChild(newHook, element);
            element = newHook;
        };
        // console.log(uwuTsxRenderManager);
        rootNode.appendChild(element);
        return [controllerInstance, rootNode];
    }
    getTemplateContent(template) {
        // if (fs.existsSync(template)) {
        //     return fs.readFileSync(template, {encoding:'utf8', flag:'r'}).toString();
        // }
        return template;
    }
    parseHTMLContent(content) {
        const parser = new DOMParser();
        const htmlDoc = parser.parseFromString(content, 'text/html');
        return htmlDoc;
    }
    static getInjectableInstance(controller, injectable) {
        let currentModule = controller.prototype.parent;
        console.log("Trying to get parent", controller, "is", controller.prototype.parent);
        while (currentModule != null) {
            const imports = currentModule.prototype.imports;
            if (Array.isArray(imports)) {
                for (const obj of imports) {
                    if (declarationCheck(obj.prototype.type, core_types_1.uwuDeclaration.Injectable) && obj == injectable) {
                        return obj;
                    }
                }
            }
            currentModule = currentModule.prototype.parent;
        }
        return null;
    }
    updateStyles(controller) {
        var _a, _b, _c;
        const head = document.querySelector('head');
        const id = `style.${controller.prototype.selector}`;
        document.querySelectorAll(`[id^="${id}"]`).forEach(style => {
            head === null || head === void 0 ? void 0 : head.removeChild(style);
        });
        if (((_c = (_b = (_a = controller.prototype) === null || _a === void 0 ? void 0 : _a.styles) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0) !== 0) {
            const newStyle = document.createElement('style');
            newStyle.innerHTML = controller.prototype.styles.join("\n");
            newStyle.id = id;
            head === null || head === void 0 ? void 0 : head.appendChild(newStyle);
        }
    }
}
exports.uwuControllerManager = uwuControllerManager;
function declarationCheck(target, compareTo) {
    return target == compareTo;
}
exports.declarationCheck = declarationCheck;
function uwuModule(config) {
    return (moduleClass) => {
        moduleClass.prototype.type = core_types_1.uwuDeclaration.Module;
        moduleClass.prototype.imports = config.imports;
        moduleClass.prototype.exports = config.exports;
        // console.log('managing', moduleClass.name);
        for (const obj of config.imports) {
            const name = `${moduleClass.name}.${obj.name}`;
            // console.log('trying to import', obj);
            switch (obj.prototype.type) {
                case core_types_1.uwuDeclaration.Router:
                    break;
                case core_types_1.uwuDeclaration.Module:
                case core_types_1.uwuDeclaration.Injectable:
                    const instance = new obj();
                    tsyringe_1.container.register(obj, { useValue: new obj() });
                    // console.log(instance, 'stored with name', name);
                    // container.registerInstance<typeof obj>(name, instance);
                    break;
                default:
                    throw new Error("Object must be uwu type (uwuModule, uwuService etc.)");
            }
        }
        for (const obj of config.exports) {
            obj.prototype.parent = moduleClass;
            // console.log('trying to export', obj);
            // switch (obj.prototype.type as uwuDeclaration) {
            //     case uwuDeclaration.Controller:
            //         const name = `${moduleClass.name}.${obj.name}`;
            //         container.register(name, {useValue: obj});
            //         console.log(`${obj} was registered`);
            //         // let t = getType<ConstructorParameters<typeof obj>>();
            //         // console.log(obj.constructor);
            //         // console.log(obj.constructor.toString());
            //         // console.log(typeof obj.constructor.arguments);
            //         // console.log(obj.prototype.constructor.arguments);
            //         break;
            //     default:
            //         throw new Error("Object must be uwu type (uwuController etc.\)");
            // }
        }
    };
}
exports.uwuModule = uwuModule;
function uwuController(config) {
    // console.log('uwuController', config.selector);
    return (objectClass) => {
        // console.log('uwuController return function');
        objectClass.prototype.type = core_types_1.uwuDeclaration.Controller;
        objectClass.prototype.selector = config.selector;
        objectClass.prototype.template = config.template;
        objectClass.prototype.styles = config.styles;
        objectClass.prototype.bindPack = {};
    };
}
exports.uwuController = uwuController;
function uwuTsxController(config) {
    // console.log('uwuController', config.selector);
    return (objectClass) => {
        // console.log('uwuController return function');
        objectClass.prototype.type = core_types_1.uwuDeclaration.ControllerTsx;
        objectClass.prototype.selector = config.selector;
        objectClass.prototype.styles = config.styles;
        const newGuid = (0, funcs_1.guid)();
        objectClass.prototype.guid = newGuid;
        objectClass.prototype.hook = config.hook(newGuid);
    };
}
exports.uwuTsxController = uwuTsxController;
function Input() {
    return function (target, propertyKey) {
        let value;
        Object.defineProperty(target, propertyKey, {
            get: () => value,
            set: (newVal) => value = newVal
        });
    };
}
exports.Input = Input;
function Inject(obj) {
    return function (target, key) {
        let service;
        Object.defineProperty(target, key, {
            get: () => service,
            set: value => service = value,
            enumerable: true
        });
        service = tsyringe_1.container.resolve(obj);
    };
}
exports.Inject = Inject;
function uwuRouter(config) {
    // console.log('uwuRouter');
    return (objectClass) => {
        // console.log('uwuRouter return function');
        objectClass.prototype.type = core_types_1.uwuDeclaration.Router;
        objectClass.prototype.routes = config.routes;
    };
}
exports.uwuRouter = uwuRouter;
function uwuInjectable() {
    return (objectClass) => {
        objectClass.prototype.type = core_types_1.uwuDeclaration.Injectable;
        // if (!container.isRegistered(objectClass)) {
        //     container.register(objectClass, { useValue: new objectClass() });
        //     console.log(objectClass, 'stored with name', objectClass.prototype.name);
        // }
        // type constructorParameters = ConstructorParameters<typeof objectClass>;
        // let vars : constructorParameters = objectClass.prototype.constructor;
    };
}
exports.uwuInjectable = uwuInjectable;
let RouterService = class RouterService {
    navigate(path) {
        let localUrl = "";
        for (let token of path) {
            if (token.startsWith('/')) {
                token = token.substring(1);
            }
            if (token.endsWith('/')) {
                token = token.substring(token.length - 2);
            }
            localUrl += token;
        }
        const result = [window.location.protocol + "/"];
        if (window.location.host.trim() != '') {
            result.push(window.location.host);
        }
        if (localUrl.trim() != '') {
            result.push(localUrl);
        }
        const newUrl = result.join('/');
        window.history.pushState({}, path[path.length - 1], newUrl);
        uwuRouteManager.generatePageByCurrentUrl();
    }
};
RouterService = __decorate([
    uwuInjectable()
], RouterService);
exports.RouterService = RouterService;
function bootstrap(module) {
    if (!Object.values(core_types_1.uwuDeclaration).includes(module.prototype.type)) {
        throw new Error("Object must be uwu type (uwuModule, uwuController, uwuService etc.\)");
    }
    if (!declarationCheck(module.prototype.type, core_types_1.uwuDeclaration.Module)) {
        throw new Error("Can bootstrap only uwuModule as enter point of app.");
    }
    window.root = module;
    window.onload = e => {
        uwuRouteManager.generatePageByCurrentUrl();
    };
    window.onpopstate = e => {
        uwuRouteManager.generatePageByCurrentUrl();
    };
}
exports.bootstrap = bootstrap;
;
//# sourceMappingURL=core.js.map